'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var reactQuery = require('react-query');
var auth = require('firebase/auth');
var auth$1 = require('@firebase/auth');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

var firestoreUnsubscribes = {};
var queryCacheUnsubscribes = {};
var eventCount = {};
function firestoreUnsubscribe(subscriptionHash) {
    var firestoreUnsubscribe = firestoreUnsubscribes[subscriptionHash];
    if (firestoreUnsubscribe && typeof firestoreUnsubscribe === "function") {
        firestoreUnsubscribe();
    }
    delete firestoreUnsubscribes[subscriptionHash];
    delete eventCount[subscriptionHash];
}
function queryCacheUnsubscribe(subscriptionHash) {
    var queryCacheUnsubscribe = queryCacheUnsubscribes[subscriptionHash];
    if (queryCacheUnsubscribe) {
        queryCacheUnsubscribe();
        delete queryCacheUnsubscribes[subscriptionHash];
    }
}
/**
 * Utility hook to subscribe to events, given a function that returns an observer callback.
 * @param queryKey The react-query queryKey
 * @param subscriptionKey A hashable key to store the subscription
 * @param subscribeFn Returns an unsubscribe function to the event
 * @param options
 * @returns
 */
function useSubscription(queryKey, subscriptionKey, subscribeFn, options) {
    var _this = this;
    var _a;
    var hashFn = (options === null || options === void 0 ? void 0 : options.queryKeyHashFn) || reactQuery.hashQueryKey;
    var subscriptionHash = hashFn(subscriptionKey);
    var queryClient = reactQuery.useQueryClient();
    var resolvePromise = function () { return null; };
    var rejectPromise = function () { return null; };
    var result = new Promise(function (resolve, reject) {
        resolvePromise = resolve;
        rejectPromise = reject;
    });
    result.cancel = function () {
        queryClient.invalidateQueries(queryKey);
    };
    if (options === null || options === void 0 ? void 0 : options.onlyOnce) {
        if (!options.fetchFn) {
            throw new Error("You must specify fetchFn if using onlyOnce mode.");
        }
        else {
            var enabled = (_a = options === null || options === void 0 ? void 0 : options.enabled) !== null && _a !== void 0 ? _a : true;
            if (enabled) {
                options
                    .fetchFn()
                    .then(resolvePromise)
                    .catch(function (err) {
                    rejectPromise(err);
                });
            }
        }
    }
    else {
        var subscribedToQueryCache = !!queryCacheUnsubscribes[subscriptionHash];
        if (!subscribedToQueryCache) {
            var queryCache = queryClient.getQueryCache();
            queryCacheUnsubscribes[subscriptionHash] = queryCache.subscribe(function (event) {
                if (!event || event.query.queryHash !== hashFn(queryKey)) {
                    return;
                }
                var query = event.query, type = event.type;
                if (type === "queryRemoved") {
                    delete eventCount[subscriptionHash];
                    queryCacheUnsubscribe(subscriptionHash);
                    firestoreUnsubscribe(subscriptionHash);
                }
                if (type === "observerAdded" || type === "observerRemoved") {
                    var observersCount = query.getObserversCount();
                    if (observersCount === 0) {
                        firestoreUnsubscribe(subscriptionHash);
                    }
                    else {
                        var isSubscribedToFirestore = !!firestoreUnsubscribes[subscriptionHash];
                        if (isSubscribedToFirestore) {
                            var cachedData = queryClient.getQueryData(queryKey);
                            var hasData = !!eventCount[subscriptionHash];
                            if (hasData) {
                                resolvePromise(cachedData !== null && cachedData !== void 0 ? cachedData : null);
                            }
                        }
                        else {
                            firestoreUnsubscribes[subscriptionHash] = subscribeFn(function (data) { return __awaiter(_this, void 0, void 0, function () {
                                var _a;
                                return __generator(this, function (_b) {
                                    (_a = eventCount[subscriptionHash]) !== null && _a !== void 0 ? _a : (eventCount[subscriptionHash] = 0);
                                    eventCount[subscriptionHash]++;
                                    if (eventCount[subscriptionHash] === 1) {
                                        resolvePromise(data || null);
                                    }
                                    else {
                                        queryClient.setQueryData(queryKey, data);
                                    }
                                    return [2 /*return*/];
                                });
                            }); });
                        }
                    }
                }
            });
        }
    }
    var queryFn = function () {
        return result;
    };
    return reactQuery.useQuery(__assign(__assign({}, options), { queryFn: queryFn, queryKey: queryKey, retry: false, staleTime: Infinity, refetchInterval: undefined, refetchOnMount: true, refetchOnWindowFocus: false, refetchOnReconnect: false }));
}

function useAuthUser(queryKey, auth, options) {
    if (options === void 0) { options = {}; }
    var subscribeFn = function (cb) {
        return auth.onAuthStateChanged(cb);
    };
    return useSubscription(queryKey, ["useAuthUser", queryKey], subscribeFn, options);
}

function useAuthIdToken(queryKey, auth, options) {
    var _this = this;
    if (options === void 0) { options = {}; }
    var subscribeFn = function (callback) {
        return auth.onIdTokenChanged(function (data) { return __awaiter(_this, void 0, void 0, function () {
            var token;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, (data === null || data === void 0 ? void 0 : data.getIdTokenResult())];
                    case 1:
                        token = _a.sent();
                        return [2 /*return*/, callback(token ? { token: token } : null)];
                }
            });
        }); });
    };
    return useSubscription(queryKey, ["useAuthIdToken", queryKey], subscribeFn, options);
}

function useAuthFetchSignInMethodsForEmail(key, auth$1, email, useQueryOptions) {
    var _a;
    return reactQuery.useQuery(__assign(__assign({}, useQueryOptions), { queryKey: (_a = useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.queryKey) !== null && _a !== void 0 ? _a : key, queryFn: function () {
            return __awaiter(this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    return [2 /*return*/, auth.fetchSignInMethodsForEmail(auth$1, email)];
                });
            });
        } }));
}

function useAuthGetRedirectResult(key, auth$1, resolver, useQueryOptions) {
    var _a;
    return reactQuery.useQuery(__assign(__assign({}, useQueryOptions), { queryKey: (_a = useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.queryKey) !== null && _a !== void 0 ? _a : key, queryFn: function () {
            return __awaiter(this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    return [2 /*return*/, auth.getRedirectResult(auth$1, resolver)];
                });
            });
        } }));
}

function useAuthApplyActionCode(auth, useMutationOptions) {
    return reactQuery.useMutation(function (oobCode) {
        return auth$1.applyActionCode(auth, oobCode);
    }, useMutationOptions);
}
function useAuthCheckActionCode(auth, useMutationOptions) {
    return reactQuery.useMutation(function (oobCode) {
        return auth$1.checkActionCode(auth, oobCode);
    }, useMutationOptions);
}
function useAuthConfirmPasswordReset(auth, useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var oobCode = _a.oobCode, newPassword = _a.newPassword;
        return auth$1.confirmPasswordReset(auth, oobCode, newPassword);
    }, useMutationOptions);
}
function useAuthCreateUserWithEmailAndPassword(auth, useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var email = _a.email, password = _a.password;
        return auth$1.createUserWithEmailAndPassword(auth, email, password);
    }, useMutationOptions);
}
function useAuthDeleteUser(useMutationOptions) {
    return reactQuery.useMutation(function (user) {
        return auth$1.deleteUser(user);
    }, useMutationOptions);
}
function useAuthLinkWithCredential(useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var user = _a.user, credential = _a.credential;
        return auth$1.linkWithCredential(user, credential);
    }, useMutationOptions);
}
function useAuthLinkWithPhoneNumber(useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var user = _a.user, phoneNumber = _a.phoneNumber, appVerifier = _a.appVerifier;
        return auth$1.linkWithPhoneNumber(user, phoneNumber, appVerifier);
    }, useMutationOptions);
}
function useAuthLinkWithPopup(useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var user = _a.user, provider = _a.provider, resolver = _a.resolver;
        return auth$1.linkWithPopup(user, provider, resolver);
    }, useMutationOptions);
}
function useAuthLinkWithRedirect(useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var user = _a.user, provider = _a.provider, resolver = _a.resolver;
        return auth$1.linkWithRedirect(user, provider, resolver);
    }, useMutationOptions);
}
function useAuthReauthenticateWithCredential(useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var user = _a.user, credential = _a.credential;
        return auth$1.reauthenticateWithCredential(user, credential);
    }, useMutationOptions);
}
function useAuthReauthenticateWithPhoneNumber(useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var user = _a.user, phoneNumber = _a.phoneNumber, appVerifier = _a.appVerifier;
        return auth$1.reauthenticateWithPhoneNumber(user, phoneNumber, appVerifier);
    }, useMutationOptions);
}
function useAuthReauthenticateWithPopup(useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var user = _a.user, provider = _a.provider, resolver = _a.resolver;
        return auth$1.reauthenticateWithPopup(user, provider, resolver);
    }, useMutationOptions);
}
function useAuthReauthenticateWithRedirect(useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var user = _a.user, provider = _a.provider, resolver = _a.resolver;
        return auth$1.reauthenticateWithRedirect(user, provider, resolver);
    }, useMutationOptions);
}
function useAuthReload(useMutationOptions) {
    return reactQuery.useMutation(function (user) {
        return auth$1.reload(user);
    }, useMutationOptions);
}
function useAuthSendEmailVerification(useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var user = _a.user, actionCodeSettings = _a.actionCodeSettings;
        return auth$1.sendEmailVerification(user, actionCodeSettings);
    }, useMutationOptions);
}
function useAuthSendPasswordResetEmail(auth, useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var email = _a.email, actionCodeSettings = _a.actionCodeSettings;
        return auth$1.sendPasswordResetEmail(auth, email, actionCodeSettings);
    }, useMutationOptions);
}
function useAuthSendSignInLinkToEmail(auth, useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var email = _a.email, actionCodeSettings = _a.actionCodeSettings;
        return auth$1.sendSignInLinkToEmail(auth, email, actionCodeSettings);
    }, useMutationOptions);
}
function useAuthSignInAnonymously(auth, useMutationOptions) {
    return reactQuery.useMutation(function () {
        return auth$1.signInAnonymously(auth);
    }, useMutationOptions);
}
function useAuthSignInWithCredential(auth, useMutationOptions) {
    return reactQuery.useMutation(function (credential) {
        return auth$1.signInWithCredential(auth, credential);
    }, useMutationOptions);
}
function useAuthSignInWithCustomToken(auth, useMutationOptions) {
    return reactQuery.useMutation(function (customToken) {
        return auth$1.signInWithCustomToken(auth, customToken);
    }, useMutationOptions);
}
function useAuthSignInWithEmailAndPassword(auth, useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var email = _a.email, password = _a.password;
        return auth$1.signInWithEmailAndPassword(auth, email, password);
    }, useMutationOptions);
}
function useAuthSignInWithEmailLink(auth, useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var email = _a.email, emailLink = _a.emailLink;
        return auth$1.signInWithEmailLink(auth, email, emailLink);
    }, useMutationOptions);
}
function useAuthSignInWithPhoneNumber(auth, useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var phoneNumber = _a.phoneNumber, appVerifier = _a.appVerifier;
        return auth$1.signInWithPhoneNumber(auth, phoneNumber, appVerifier);
    }, useMutationOptions);
}
function useAuthSignInWithPopup(auth, useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var provider = _a.provider, resolver = _a.resolver;
        return auth$1.signInWithPopup(auth, provider, resolver);
    }, useMutationOptions);
}
function useAuthSignInWithRedirect(auth, useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var provider = _a.provider, resolver = _a.resolver;
        return auth$1.signInWithRedirect(auth, provider, resolver);
    }, useMutationOptions);
}
function useAuthSignOut(auth, useMutationOptions) {
    return reactQuery.useMutation(function () {
        return auth$1.signOut(auth);
    }, useMutationOptions);
}
function useAuthUnlink(useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var user = _a.user, providerId = _a.providerId;
        return auth$1.unlink(user, providerId);
    }, useMutationOptions);
}
function useAuthUpdateCurrentUser(auth, useMutationOptions) {
    return reactQuery.useMutation(function (user) {
        return auth$1.updateCurrentUser(auth, user);
    }, useMutationOptions);
}
function useAuthUpdateEmail(useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var user = _a.user, newEmail = _a.newEmail;
        return auth$1.updateEmail(user, newEmail);
    }, useMutationOptions);
}
function useAuthUpdatePassword(useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var user = _a.user, newPassword = _a.newPassword;
        return auth$1.updatePassword(user, newPassword);
    }, useMutationOptions);
}
function useAuthUpdatePhoneNumber(useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var user = _a.user, credential = _a.credential;
        return auth$1.updatePhoneNumber(user, credential);
    }, useMutationOptions);
}
function useAuthUpdateProfile(useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var user = _a.user, update = __rest(_a, ["user"]);
        return auth$1.updateProfile(user, update);
    }, useMutationOptions);
}
function useAuthVerifyBeforeUpdateEmail(useMutationOptions) {
    return reactQuery.useMutation(function (_a) {
        var user = _a.user, newEmail = _a.newEmail, actionCodeSettings = _a.actionCodeSettings;
        return auth$1.verifyBeforeUpdateEmail(user, newEmail, actionCodeSettings);
    }, useMutationOptions);
}
function useAuthVerifyPasswordResetCode(auth, useMutationOptions) {
    return reactQuery.useMutation(function (code) {
        return auth$1.verifyPasswordResetCode(auth, code);
    }, useMutationOptions);
}

exports.useAuthApplyActionCode = useAuthApplyActionCode;
exports.useAuthCheckActionCode = useAuthCheckActionCode;
exports.useAuthConfirmPasswordReset = useAuthConfirmPasswordReset;
exports.useAuthCreateUserWithEmailAndPassword = useAuthCreateUserWithEmailAndPassword;
exports.useAuthDeleteUser = useAuthDeleteUser;
exports.useAuthFetchSignInMethodsForEmail = useAuthFetchSignInMethodsForEmail;
exports.useAuthGetRedirectResult = useAuthGetRedirectResult;
exports.useAuthIdToken = useAuthIdToken;
exports.useAuthLinkWithCredential = useAuthLinkWithCredential;
exports.useAuthLinkWithPhoneNumber = useAuthLinkWithPhoneNumber;
exports.useAuthLinkWithPopup = useAuthLinkWithPopup;
exports.useAuthLinkWithRedirect = useAuthLinkWithRedirect;
exports.useAuthReauthenticateWithCredential = useAuthReauthenticateWithCredential;
exports.useAuthReauthenticateWithPhoneNumber = useAuthReauthenticateWithPhoneNumber;
exports.useAuthReauthenticateWithPopup = useAuthReauthenticateWithPopup;
exports.useAuthReauthenticateWithRedirect = useAuthReauthenticateWithRedirect;
exports.useAuthReload = useAuthReload;
exports.useAuthSendEmailVerification = useAuthSendEmailVerification;
exports.useAuthSendPasswordResetEmail = useAuthSendPasswordResetEmail;
exports.useAuthSendSignInLinkToEmail = useAuthSendSignInLinkToEmail;
exports.useAuthSignInAnonymously = useAuthSignInAnonymously;
exports.useAuthSignInWithCredential = useAuthSignInWithCredential;
exports.useAuthSignInWithCustomToken = useAuthSignInWithCustomToken;
exports.useAuthSignInWithEmailAndPassword = useAuthSignInWithEmailAndPassword;
exports.useAuthSignInWithEmailLink = useAuthSignInWithEmailLink;
exports.useAuthSignInWithPhoneNumber = useAuthSignInWithPhoneNumber;
exports.useAuthSignInWithPopup = useAuthSignInWithPopup;
exports.useAuthSignInWithRedirect = useAuthSignInWithRedirect;
exports.useAuthSignOut = useAuthSignOut;
exports.useAuthUnlink = useAuthUnlink;
exports.useAuthUpdateCurrentUser = useAuthUpdateCurrentUser;
exports.useAuthUpdateEmail = useAuthUpdateEmail;
exports.useAuthUpdatePassword = useAuthUpdatePassword;
exports.useAuthUpdatePhoneNumber = useAuthUpdatePhoneNumber;
exports.useAuthUpdateProfile = useAuthUpdateProfile;
exports.useAuthUser = useAuthUser;
exports.useAuthVerifyBeforeUpdateEmail = useAuthVerifyBeforeUpdateEmail;
exports.useAuthVerifyPasswordResetCode = useAuthVerifyPasswordResetCode;
